###### R's must-knows ######

#always remeber to set up your working directory before starting
#any analyses

setwd("C:/R")

#R works on vectors so everything, even single number is a vector
#vectors on higher-order structures such as matrices can be
#added, multiplied, etc. in the same way we do this with simple
#numbers

vector <- 1:20
vector
matr <- matrix(vector,4,5)
matr
matr*3.5
sqrt(vector)

#several functions work on vectors in the way that they extract
#some simple number from them

mean(vector)
sum(vector)
colSums(matr)
rowMeans(matr)

#we can name elements of matices and vectors

colnames(matr) <- LETTERS[1:5]
rownames(matr) <- LETTERS[15:18]
matr

#other useful types of data structures are lists
#they can contain data of different types (text,numbers,etc)
#whereas vector convert data to one type or display error

vector <- c(1,2,"word",c(22,33))
list1 <- list(1,2,"word",c(22,33))
vector
list1

#mostly used data structures in R are data frames
#you can load your own data to R like this:
#mydata <- read.table("file",sep="\t",head=T) for .-delimited decimals
#mydata <- read.delim2("file",sep="\t",head=T) for ,-delimited decimals
#in both examples data are separated by Tabs and first row contains names
#you can also use built-in data from R
data(ChickWeight)
summary(ChickWeight)
names(ChickWeight)

#remember that in R rows are indexed first, then columns
#you can also refer to columns using their names
#here we simultaneously select columns and then extract some subset
#of their data using indexes on vectors
ChickWeight[,1][1:10]
ChickWeight[2,]
ChickWeight$Diet[205:220]

matr[,2] #access 2nd column of the matrix but loosing its dimensions
matr[,2,drop=F] #keep right dimensions

#sometimes instead of numerical indexes logical subscripts can be used

ChickWeight[ChickWeight$Diet=="A",][1:20,]
ChickWeight[ChickWeight$Diet=="A"&ChickWeight$Time==18,]
ix<-which(ChickWeight$Diet=="B"&(ChickWeight$Time==10|ChickWeight$Time==12))
ix
ChickWeight[ix,]

#as you probably suspect negative indexes result in deletion of
#respective columns/rows

#as you've noticed some data are in the form of categorical variables
#called factors

ChickWeight$Diet[1:10]
levels(ChickWeight$Diet)
levels(ChickWeight$Diet)<-LETTERS[1:4]
summary(ChickWeight)

#you can use a set of functions to change and query the types
#of objects

as.character(ChickWeight$Diet)[205:220]
as.numeric(ChickWeight$Chick)[205:220]
is.factor(ChickWeight$Diet)
is.numeric(ChickWeight$Chick)

#NA is a special type of data where there's no value
#to test if sth is NA you should use is.na(x) instead of x==NA

#manipulating large datasets is easy thanks to several automating
#functions

#1 combine two matrices/vectors/data frames

cbind(matr,4*matr) #column-wise
rbind(matr,4*matr) #row-wise

#2 apply some function column- or row-wise in matrix or data frame

apply(matr,1,var)
apply(matr,2,mean)

#3 apply a function group-wise

tapply(ChickWeight$weight,ChickWeight$Diet,mean)
tapply(ChickWeight$weight,ChickWeight$Chick,var)

aggregate(ChickWeight$weight,list(ChickWeight$Diet),mean)

#4 build a contingency table

table(ChickWeight$Diet,ChickWeight$Chick)
#---------------------------------------
#houskeeping
rm(matr) #removes selected object
attach(ChickWeight) #attaches objects so that variables can be called directly
detach(ChickWeight) #detaches object
#attach is useful but can be dangerous - you can use with() instead
attach(ChickWeight)
ix<-which(Diet=="B"&(Time==10|Time==12))
detach(ChickWeight)
#OR
with(ChickWeight,ix<-which(Diet=="B"&(Time==10|Time==12)))
#not very longer and much safer ;)

library(lme4) #loads library
detach(package:lme4) #unloads library

save(file="filename") #saves current workspace
savehistory(file="filename2") #saves history of commands

ls() #lists all objects
#---------------------------------------
#generating useful data
seq(1,10,0.5)
seq(10,1,-1)

rep(c(1,2),times=3)
rep(c(1,2),each=3)
rep(c(1,2),each=3,times=3)

gl(4,5,labels=c("A","B","C","D"))
sample(1:100,10)
sample(1:10,10,rep=T)

#operations on distributions
rnorm(10,mean=2,sd=sqrt(2)) #10 numbers from normal distribution
pnorm(2,mean=2,sd=sqrt(2)) #proportion of distribution to to X
qnorm(0.5,mean=2,sd=sqrt(2)) #quantile of distribution for P
dnorm(0.5, mean=2, sd=sqrt(2)) #density of distribution at X

#most popular distribution are:
#norm - gaussian
#pois - poisson
#binom - binomial
#gamma - gamma
#beta - beta
#chisq - Chi squared
#unif - uniform
#logis - logistic
#lnrom - log-normal
#---------------------------------------
#basic analyses and interpretation
#linear model - gaussian data
m1<-lm(weight~Time*Diet,data=ChickWeight)
#here we add higher-order terms to allow for curvilinear trends
m2<-lm(weight~poly(Time,2,raw=T)*Diet,data=ChickWeight)
summary(m1)
summary.aov(m1)
summary(m2)
summary.aov(m2)
#note that results in LM output are presented as intercept (global mean
#containing alphabetically first levels from fixed effects, and for
#continuous predictors = 0) and then slopes for covariates and differences
#between intercept and each level of categorical predictors

#if you want - youcan set-up contrasts to make specific comparisons
#let's assume we'd like to compare: diet 1 with 2,3,4 and diet 3 and 4
contrasts(ChickWeight$Diet)<-cbind(c(-3,1,1,1),c(0,0,-1,1))
contrasts(ChickWeight$Diet)
#R generated additional contrast to ensure orthogonality of comparisons
#we can force two constrasts but be careful interpreting contasts that are
#non-orthogonal!
contrasts(ChickWeight$Diet,how.many=2)<-cbind(c(-3,1,1,1),c(0,0,-1,1))
contrasts(ChickWeight$Diet)
m3<-lm(weight~Time*Diet,data=ChickWeight)
summary(m3)
summary.aov(m3)

#contrasts are compatible with most modelling functions
#we can use them even with mixed models
library(MCMCglmm)
m4 <- MCMCglmm(weight~Diet, random=~Chick,data=ChickWeight,verbose=F)
summary(m4) #2 contrasts instead of 3 levels for Diet 2,3,4
